/*
 * Copyright (c) 2010, Noel Lemouel
 * All rights reserved.
 */

#include "tga.h"
#include "sprite.h"

s32 sprite_blit_16(fb_t* fb, sprite_t* sprite, u16 x, u16 y)
{
	u32 i,j;
	s16* sc;
	s16* ds;

	sc = (s16*)sprite->base;
	ds = (s16*)fb->base+x+(y*fb->w);

	for(i=0; i<sprite->h; i++)
	{
		for(j=0; j<sprite->w; j++)
		{
			*ds++ = *sc++;
		}
		ds+=fb->w-sprite->w;
	}

	return 0;
}

int sprite_blit_alpha_16(fb_t* fb, sprite_t* sprite, int x, int y, unsigned char a)
{
	int i,j;
	unsigned char rgb_24_00[3];
	unsigned char rgb_24_01[3];
	unsigned char rgb_24_02[3];
	unsigned short* sc_16;
	unsigned short* ds_16;

	sc_16 = (unsigned short*)sprite->base;
	ds_16 = (unsigned short*)fb->base+x+(y*fb->w);

	for(i=0; i<sprite->h; i++)
	{
		for(j=0; j<sprite->w; j++)
		{
			/* fb */
			rgb_24_00[0] = (*ds_16 & 0xf800) >> 8;
			rgb_24_00[1] = (*ds_16 & 0x07e0) >> 3;
			rgb_24_00[2] = (*ds_16 & 0x001f) << 3;

			/* sprite */
			rgb_24_01[0] = (*sc_16 & 0xf800) >> 8;
			rgb_24_01[1] = (*sc_16 & 0x07e0) >> 3;
			rgb_24_01[2] = (*sc_16 & 0x001f) << 3;

			/* alpha compositing */

			rgb_24_02[0] = ( ((255-a)*rgb_24_01[0]) + (a*rgb_24_00[0]) ) / 255;
			rgb_24_02[1] = ( ((255-a)*rgb_24_01[1]) + (a*rgb_24_00[1]) ) / 255;
			rgb_24_02[2] = ( ((255-a)*rgb_24_01[2]) + (a*rgb_24_00[2]) ) / 255;

			*ds_16 = 0x00 | ((rgb_24_02[0] & 0xf8) << 8);
			*ds_16 |= (rgb_24_02[1] & 0xfc) << 3;
			*ds_16 |= (rgb_24_02[2] & 0xfe) >> 3;

			ds_16++;
			sc_16++;

		}
		ds_16+=fb->w-sprite->w;
	}

	return 0;
}

s32 sprite_area_blit_16(fb_t* fb, sprite_t* sprite, area_t* area, u16 x, u16 y)
{
	u32 i,j;
	u16* sc;
	u16* ds;
	
	sc = (u16*)sprite->base+area->x+(area->y*sprite->w);
	ds = (u16*)fb->base+x+(y*fb->w);

	for(i=0; i<area->h; i++)
	{
		for(j=0; j<area->w; j++)
		{
			*ds++ = *sc++;
		}
		sc+=sprite->w-area->w;
		ds+=fb->w-area->w;
	}

	return 0;
}

