/*
 * Copyright (c) 2010, Noel Lemouel
 * All rights reserved.
 */

#include <stdio.h>

#include "tga.h"
#include "sprite.h"

s32 tga_info(u8* tga_img, tga_info_t* tga_info)
{
	struct tga_header* tga_header_00;
	struct tga_img_spec* tga_img_spec_00;

	tga_header_00 = (struct tga_header*)tga_img;
	tga_img_spec_00 = (struct tga_img_spec*)tga_header_00->img_spec;

/*	if(tga_header_00->img_type != TGA_TYPE_TRUE_COLOR) return -1; */

	tga_info->base = (u8*) tga_img + sizeof(struct tga_header) + tga_header_00->id_len;

	tga_info->w = (u16)0x00 | tga_img_spec_00->w[0];
	tga_info->w |= tga_img_spec_00->w[1] << 8;

	tga_info->h = (u16)0x00 | tga_img_spec_00->h[0];
	tga_info->h |= tga_img_spec_00->h[1] << 8;

	tga_info->bpp = tga_img_spec_00->bpp;

	tga_info->hv = tga_img_spec_00->dsc & 0x20;

	tga_info->type = tga_header_00->img_type;

	return 0;
}

void tga24_to_sprite16(tga_info_t* tga_24, sprite_t* sprite_16)
{
	u32 i, j, k, l;

	u16* pxl_16;
	u8* pxl_24;

	if(tga_24->hv)
	{
		pxl_16 = (u16*)sprite_16->base;
	}else{
		pxl_16 = (u16*)sprite_16->base;
		pxl_16 += (tga_24->w * (tga_24->h-1));
	}
	pxl_24 = tga_24->base;

	sprite_16->w = tga_24->w;
	sprite_16->h = tga_24->h;
	sprite_16->bpp = 16;

	i = tga_24->w*tga_24->h;
	k = 0;

	if((tga_24->type == TGA_TYPE_TRUE_COLOR) && (tga_24->bpp == 24))
	{
		while(i--)
		{
			*pxl_16 = rgb24_to_16(pxl_24[2], pxl_24[1], pxl_24[0]);
			pxl_16++; k++; if(!(k%tga_24->w) && !tga_24->hv) pxl_16-=tga_24->w<<1;
			pxl_24+=3;
		}
	}else if((tga_24->type == TGA_TYPE_RLE_TRUE_COLOR) && (tga_24->bpp == 24)){
		while(k < i)
		{
			if(pxl_24[0] & 0x80){
				/* encoded packets */
				for(j=0; j<=(pxl_24[0] & 0x7f); j++)
				{
					*pxl_16 = rgb24_to_16(pxl_24[3], pxl_24[2], pxl_24[1]);
					pxl_16++; k++; if(!(k%tga_24->w) && !tga_24->hv) pxl_16-=tga_24->w<<1;
					if(k > i) return;
				}
				pxl_24+=4;
			}else{
				/* raw packets */
				l = pxl_24[0] & 0x7f;
				pxl_24++;
				for(j=0; j<=l; j++)
				{
					*pxl_16 = rgb24_to_16(pxl_24[2], pxl_24[1], pxl_24[0]);
					pxl_16++; k++; if(!(k%tga_24->w) && !tga_24->hv) pxl_16-=tga_24->w<<1;
					if(k > i) return;
					pxl_24+=3;
				}
			}
		}
	}else{
		printf("tga24_to_sprite16: unsuppoted TGA format [%d/%dbpp]\n", tga_24->type, tga_24->bpp);
		sprite_16->w = 0;
		sprite_16->h = 0;
		sprite_16->bpp = 0;
	}
}

