/*
 * Copyright (c) 2011, Noel Lemouel
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#ifdef MACOSX
#include "SDL/include/SDL.h"
#else
#include "SDL.h"
#endif

#include "draw.h"
#include "sprite.h"
#include "tga.h"

SDL_Surface *screen;

#define FB_W 512
#define FB_H 512

char *fname[] = {
   "tga/sample_0.tga",
   "tga/sample_1.tga",
   "tga/sample_2.tga",
   "tga/sample_3.tga"
};

/* forward */
static int sc_open(FILE**, const char*, struct stat*);

void render()
{   
   if (SDL_MUSTLOCK(screen)) 
      SDL_UnlockSurface(screen);

   SDL_UpdateRect(screen, 0, 0, FB_W, FB_H);    

   if (SDL_MUSTLOCK(screen))
      if (SDL_LockSurface(screen) < 0) 
         return;
}

int main(int argc, char *argv[])
{
   unsigned int i;
   unsigned short* fb_16;
   unsigned char* tga;
   unsigned char* sprite_mem_16;

   fb_t fb;
   sprite_t sprite_16;
   tga_info_t _tga_info;

   FILE* fd;
   struct stat info;

   /* SDL init. */

   atexit(SDL_Quit);

   screen = SDL_SetVideoMode(FB_W, FB_H, 16, SDL_SWSURFACE);

   if ( screen == NULL ) 
   {
      fprintf(stderr, "Unable to set %dx%d video: %s\n", FB_W, FB_H, SDL_GetError());
      exit(1);
   }

   SDL_WM_SetCaption("v_tga", NULL);

   fb.w = FB_W;
   fb.h = FB_H;
   fb.bpp = 16;
   fb.base = (unsigned char*)(screen->pixels);

   /* clear screen */

   fb_16 = (unsigned short*)(screen->pixels);
   for(i=0; i<FB_W*FB_H; i++)
   {
      fb_16[i] = rgb24_to_16(0x00, 0x00, 0xff);
   }

   /* blit tga files content */

   for(i=0; i<4; i++)
   {
      if( sc_open(&fd, fname[i], &info) ) return -1;

      /* load tga file */

      tga = malloc(info.st_size);
      if(!tga){
         printf("error: [alloc. %d] failed\n", (int)info.st_size);
         fclose(fd);
         return -1;
      }else{
         // printf("[alloc. %d]\n", (int)info.st_size);
      }

      fread(tga, (int)info.st_size, 1, fd);   

      fclose(fd);

      /* ... */

      tga_info(tga, &_tga_info);

      printf("%s\n", fname[i]);
      printf("w: %d\n", _tga_info.w);
      printf("h: %d\n", _tga_info.h);
      printf("hv: %d\n", _tga_info.hv >> 5);
      printf("bpp: %d\n", _tga_info.bpp);
      printf("type: %d\n", _tga_info.type);

      sprite_mem_16 = (unsigned char*)malloc(_tga_info.w * _tga_info.h * 2);
      if(!sprite_mem_16){
         printf("error: [alloc. %d] failed\n", (int)(_tga_info.w * _tga_info.h * 2));
         free(tga);
         return -1;
      }else{
         // printf("[alloc. %d]\n", (int)(_tga_info.w * _tga_info.h * 2));
      }

      sprite_16.base = sprite_mem_16;

      tga24_to_sprite16(&_tga_info, &sprite_16);


      /* blit sprite_16 */

      switch(i)
      {
         case 0:
            sprite_blit_16(&fb, &sprite_16, 0, 0);
            break;
         case 1:
            sprite_blit_16(&fb, &sprite_16, 256, 0);
            break;
         case 2:
            sprite_blit_16(&fb, &sprite_16, 0, 256);
            break;
         case 3:
            sprite_blit_16(&fb, &sprite_16, 256, 256);
            break;
         default:
            printf("?\n");
            break;
      }

      free(tga);
      free(sprite_mem_16);
   }

   /* look for events */

   while(1)
   {
      render();
      SDL_Delay(100);

      SDL_Event event;
      while (SDL_PollEvent(&event)) 
      {
         switch (event.type) 
         {
            case SDL_KEYDOWN:
               break;
            case SDL_KEYUP:
               if (event.key.keysym.sym == SDLK_ESCAPE)
                  return 0;
               break;
            case SDL_QUIT:
               return 0;
         }
      }
   }

   return 0;
}

static int sc_open(FILE** fd, const char* name, struct stat* info)
{
   if (stat(name, info) == -1)
   {
      if (errno == ENOENT)
      {
         (void) fprintf( stderr, "error: '%s' does not exist\r\n", name );
      }else{
         (void) fprintf( stderr, "error: stat() fails for '%s'\r\n", name );
      }
      return -1;
   }

   /* checking S_ISLNK(m) is not required since 'stat' follows sym. links */
   if (!S_ISREG(info->st_mode))
   {
      (void) fprintf( stderr, "error: '%s' is a not a regular file (or a sym. link)\r\n", name);
      return -1;
   }

   if ((*fd = fopen(name, "rb")) == NULL)
   {
      (void) fprintf( stderr, "error: '%s' open failed\r\n", name );
      return -1;
   }

   /* printf("size: %d\n", (int)info->st_size); */

   return 0;   /* no error */
}

